      SUBROUTINE ZNGETS ( ISHIFT, WHICH, KEV, NP, RITZ, BOUNDS)
C---------------------------------------------------------------------
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ALGELINE  DATE 17/02/2003   AUTEUR NICOLAS O.NICOLAS 
C ======================================================================
C COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
C (AT YOUR OPTION) ANY LATER VERSION.                                   
C                                                                       
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
C                                                                       
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
C   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.         
C ======================================================================
C ======================================================================
C
C     SUBROUTINE ARPACK CALCULANT NP SHIFTS DU RESTART DE IRAM.
C---------------------------------------------------------------------
C\BEGINDOC
C
C\NAME: ZNGETS
C
C\DESCRIPTION: 
C  GIVEN THE EIGENVALUES OF THE UPPER HESSENBERG MATRIX H,
C  COMPUTES THE NP SHIFTS AMU THAT ARE ZEROS OF THE POLYNOMIAL OF 
C  DEGREE NP WHICH FILTERS OUT COMPONENTS OF THE UNWANTED EIGENVECTORS
C  CORRESPONDING TO THE AMU'S BASED ON SOME GIVEN CRITERIA.
C
C  NOTE: CALL THIS EVEN IN THE CASE OF USER SPECIFIED SHIFTS IN ORDER
C  TO SORT THE EIGENVALUES, AND ERROR BOUNDS OF H FOR LATER USE.
C
C\USAGE:
C  CALL ZNGETS
C      ( ISHIFT, WHICH, KEV, NP, RITZ, BOUNDS )
C
C\ARGUMENTS
C  ISHIFT  INTEGER.  (INPUT)
C          METHOD FOR SELECTING THE IMPLICIT SHIFTS AT EACH ITERATION.
C          ISHIFT = 0: USER SPECIFIED SHIFTS
C          ISHIFT = 1: EXACT SHIFT WITH RESPECT TO THE MATRIX H.
C
C  WHICH   CHARACTER*2.  (INPUT)
C          SHIFT SELECTION CRITERIA.
C          'LM' -> WANT THE KEV EIGENVALUES OF LARGEST MAGNITUDE.
C          'SM' -> WANT THE KEV EIGENVALUES OF SMALLEST MAGNITUDE.
C          'LR' -> WANT THE KEV EIGENVALUES OF LARGEST REAL PART.
C          'SR' -> WANT THE KEV EIGENVALUES OF SMALLEST REAL PART.
C          'LI' -> WANT THE KEV EIGENVALUES OF LARGEST IMAGINARY PART.
C          'SI' -> WANT THE KEV EIGENVALUES OF SMALLEST IMAGINARY PART.
C
C  KEV     INTEGER.  (INPUT)
C          THE NUMBER OF DESIRED EIGENVALUES.
C
C  NP      INTEGER.  (INPUT)
C          THE NUMBER OF SHIFTS TO COMPUTE.
C
C  RITZ    COMPLEX*16 ARRAY OF LENGTH KEV+NP.  (INPUT/OUTPUT)
C          ON INPUT, RITZ CONTAINS THE THE EIGENVALUES OF H.
C          ON OUTPUT, RITZ ARE SORTED SO THAT THE UNWANTED
C          EIGENVALUES ARE IN THE FIRST NP LOCATIONS AND THE WANTED
C          PORTION IS IN THE LAST KEV LOCATIONS.  WHEN EXACT SHIFTS ARE 
C          SELECTED, THE UNWANTED PART CORRESPONDS TO THE SHIFTS TO 
C          BE APPLIED. ALSO, IF ISHIFT .EQ. 1, THE UNWANTED EIGENVALUES
C          ARE FURTHER SORTED SO THAT THE ONES WITH LARGEST RITZ VALUES
C          ARE FIRST.
C
C  BOUNDS  COMPLEX*16 ARRAY OF LENGTH KEV+NP.  (INPUT/OUTPUT)
C          ERROR BOUNDS CORRESPONDING TO THE ORDERING IN RITZ.
C
C  
C
C\ENDDOC
C
C-----------------------------------------------------------------------
C
C\BEGINLIB
C
C\LOCAL VARIABLES:
C     XXXXXX  COMPLEX*16
C
C\ROUTINES CALLED:
C     ZSORTC  ARPACK SORTING ROUTINE.
C     IVOUT   ARPACK UTILITY ROUTINE THAT PRINTS INTEGERS.
C     SECOND  ARPACK UTILITY ROUTINE FOR TIMING.
C     ZVOUT   ARPACK UTILITY ROUTINE THAT PRINTS VECTORS.
C
C\AUTHOR
C     DANNY SORENSEN               PHUONG VU
C     RICHARD LEHOUCQ              CRPC / RICE UNIVERSITY
C     DEPT. OF COMPUTATIONAL &     HOUSTON, TEXAS
C     APPLIED MATHEMATICS 
C     RICE UNIVERSITY           
C     HOUSTON, TEXAS 
C
C\SCCS INFORMATION: @(#)
C FILE: NGETS.F   SID: 2.2   DATE OF SID: 4/20/96   RELEASE: 2
C
C\REMARKS
C     1. THIS ROUTINE DOES NOT KEEP COMPLEX CONJUGATE PAIRS OF
C        EIGENVALUES TOGETHER.
C
C\ENDLIB
C
C-----------------------------------------------------------------------
C CORPS DU PROGRAMME
      IMPLICIT NONE

C     %-----------------------------%
C     | INCLUDE FILES FOR DEBUGGING |
C     %-----------------------------%

      INTEGER LOGFIL, NDIGIT, MGETV0,
     &  MNAUPD, MNAUP2, MNAITR, MNEIGH, MNAPPS, MNGETS, MNEUPD
      COMMON /DEBUG/
     &  LOGFIL, NDIGIT, MGETV0,
     &  MNAUPD, MNAUP2, MNAITR, MNEIGH, MNAPPS, MNGETS, MNEUPD

C     %------------------%
C     | SCALAR ARGUMENTS |
C     %------------------%

      CHARACTER*2 WHICH
      INTEGER    ISHIFT, KEV, NP
C
C     %-----------------%
C     | ARRAY ARGUMENTS |
C     %-----------------%
C
      COMPLEX*16 BOUNDS(KEV+NP), RITZ(KEV+NP)
C
C     %------------%
C     | PARAMETERS |
C     %------------%
C
C
C     %---------------%
C     | LOCAL SCALARS |
C     %---------------%
C
      INTEGER    MSGLVL
C
C
C     %-----------------------%
C     | EXECUTABLE STATEMENTS |
C     %-----------------------%
C
C     %-------------------------------%
C     | INITIALIZE TIMING STATISTICS  |
C     | & MESSAGE LEVEL FOR DEBUGGING |
C     %-------------------------------%
C 
      MSGLVL = MNGETS
C 
      CALL ZSORTC (WHICH, .TRUE., KEV+NP, RITZ, BOUNDS)
C     
      IF ( ISHIFT .EQ. 1 ) THEN
C     
C        %-------------------------------------------------------%
C        | SORT THE UNWANTED RITZ VALUES USED AS SHIFTS SO THAT  |
C        | THE ONES WITH LARGEST RITZ ESTIMATES ARE FIRST        |
C        | THIS WILL TEND TO MINIMIZE THE EFFECTS OF THE         |
C        | FORWARD INSTABILITY OF THE ITERATION WHEN THE SHIFTS  |
C        | ARE APPLIED IN SUBROUTINE ZNAPPS.                     |
C        | BE CAREFUL AND USE 'SM' SINCE WE WANT TO SORT BOUNDS! |
C        %-------------------------------------------------------%
C     
         CALL ZSORTC ( 'SM', .TRUE., NP, BOUNDS, RITZ )
C
      END IF
C     
C
      IF (MSGLVL .GT. 0) THEN
         CALL IVOUT (LOGFIL, 1, KEV, NDIGIT, '_NGETS: KEV IS')
         CALL IVOUT (LOGFIL, 1, NP, NDIGIT, '_NGETS: NP IS')
         CALL ZVOUT (LOGFIL, KEV+NP, RITZ, NDIGIT,
     &        '_NGETS: EIGENVALUES OF CURRENT H MATRIX ')
         CALL ZVOUT (LOGFIL, KEV+NP, BOUNDS, NDIGIT, 
     &      '_NGETS: RITZ ESTIMATES OF THE CURRENT KEV+NP RITZ VALUES')
      END IF
C     
C     
C     %---------------%
C     | END OF ZNGETS |
C     %---------------%
C     
      END
