        SUBROUTINE LC0000 ( FAMI,KPG,KSP,NDIM,TYPMOD,IMATE,COMPOR,CRIT,
     &                      INSTAM,INSTAP,EPSM,DEPS,SIGM,VIM, 
     &              OPTION,TAMPON,ANGMAS,CP,NUMLC,TEMPD,TEMPF,TREF,
     &                      SIGP,VIP,DSIDEP,ICOMP, NVI, CODRET)
        IMPLICIT NONE
C       ================================================================
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ALGORITH  DATE 07/02/2011   AUTEUR MEUNIER S.MEUNIER 
C ======================================================================
C COPYRIGHT (C) 1991 - 2011  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
C (AT YOUR OPTION) ANY LATER VERSION.                                   
C                                                                       
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
C                                                                       
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
C   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.         
C ======================================================================
C TOLE CRP_20
C TOLE CRP_21
C RESPONSABLE PROIX J-M.PROIX
C       ================================================================
C       INTEGRATION DE LOIS DE COMPORTEMENT ELASTO PLASTIQUE ET VISCO
C       PLASTIQUE
C               AVEC    . N VARIABLES INTERNES
C                       . UNE FONCTION SEUIL ELASTIQUE
C
C       INTEGRATION DES CONTRAINTES           = SIG(T+DT)
C       INTEGRATION DES VARIABLES INTERNES    = VIN(T+DT)
C       ET CALCUL DU JACOBIEN ASSOCIE         = DS/DE(T+DT) OU DS/DE(T)
C
C       - SI CRIT(5) = -N  EN CAS DE NON-CONVERGENCE LOCALE ON EFFECTUE
C                          UN REDECOUPAGE DU PAS DE TEMPS EN N PALIERS
C                          L ORDRE D EXECUTION ETANT REMONTE EN ARGUMENT
C                          DANS REDECE, APPELE PAR NMCOMP AVANT PLASTI
C       - SI CRIT(5) = -1,0,1  PAS DE REDECOUPAGE DU PAS DE TEMPS
C       - SI CRIT(5) = +N  ON EFFECTUE UN REDECOUPAGE DU PAS DE TEMPS
C                          EN N PALIERS A CHAQUE APPEL DE REDECE
C                          LE PREMIER APPEL DE PLASTI SERT A
C                          L'INITIALISATION DE NVI
C       SI APRES REDECOUPAGE ON ABOUTIT A UN CAS DE NON CONVERGENCE, ON
C       REDECOUPE A NOUVEAU LE PAS DE TEMPS, EN 2*N PALIERS
C       ================================================================
C
C       PLASTI  ALGORITHME D INTEGRATION ELASTO PLASTIQUE
C       LCXXXX  ROUTINES UTILITAIRES (VOIR LE DETAIL DANS PLASTI)
C       ================================================================
C       ARGUMENTS
C
C       IN      FAMI    FAMILLE DE POINT DE GAUSS (RIGI,MASS,...)
C       IN      KPG,KSP NUMERO DU (SOUS)POINT DE GAUSS
C       IN      NDIM    DIMENSION DE L ESPACE (3D=3,2D=2,1D=1)
C               TYPMOD  TYPE DE MODELISATION
C               IMATE    ADRESSE DU MATERIAU CODE
C               COMPOR    COMPORTEMENT DE L ELEMENT
C                       COMPOR(1) = RELATION DE COMPORTEMENT (VMIS_...)
C                       COMPOR(2) = NB DE VARIABLES INTERNES
C                       COMPOR(3) = TYPE DE DEFORMATION (PETIT,GREEN...)
C               OPTION     OPTION DE CALCUL A FAIRE
C                             'RIGI_MECA_TANG'> DSIDEP(T)
C                             'FULL_MECA'     > DSIDEP(T+DT) , SIG(T+DT)
C                             'RAPH_MECA'     > SIG(T+DT)
C               CRIT    CRITERES  LOCAUX
C                       CRIT(1) = NOMBRE D ITERATIONS MAXI A CONVERGENCE
C                                 (ITER_INTE_MAXI == ITECREL)
C                       CRIT(2) = TYPE DE JACOBIEN A T+DT
C                                 (TYPE_MATR_COMP == MACOMP)
C                                 0 = EN VITESSE     > SYMETRIQUE
C                                 1 = EN INCREMENTAL > NON-SYMETRIQUE
C                       CRIT(3) = VALEUR DE LA TOLERANCE DE CONVERGENCE
C                                 (RESI_INTE_RELA == RESCREL)
C                       CRIT(5) = NOMBRE D'INCREMENTS POUR LE
C                                 REDECOUPAGE LOCAL DU PAS DE TEMPS
C                                 (ITER_INTE_PAS == ITEDEC)
C                                 0 = PAS DE REDECOUPAGE
C                                 N = NOMBRE DE PALIERS
C               TAMPON  TABLEAUX DES ELEMENTS GEOMETRIQUES SPECIFIQUES
C                       AUX LOIS DE COMPORTEMENT (DIMENSION MAXIMALE
C                       FIXEE EN DUR)
C               INSTAM   INSTANT T
C               INSTAP   INSTANT T+DT
C               EPSM   DEFORMATION TOTALE A T
C               DEPS   INCREMENT DE DEFORMATION TOTALE
C               SIGM    CONTRAINTE A T
C               VIM    VARIABLES INTERNES A T    + INDICATEUR ETAT T
C    ATTENTION  VIM    VARIABLES INTERNES A T MODIFIEES SI REDECOUPAGE
C       OUT     SIGP    CONTRAINTE A T+DT
C               VIP    VARIABLES INTERNES A T+DT + INDICATEUR ETAT T+DT
C               DSIDEP    MATRICE DE COMPORTEMENT TANGENT A T+DT OU T
C       ----------------------------------------------------------------
        INTEGER         IMATE,NDIM,NDT,NDI,NVI,KPG,KSP
C
        REAL*8          CRIT(*), ANGMAS(3)
        REAL*8          INSTAM,     INSTAP,    TEMPD,   TEMPF  , TREF
        REAL*8          TAMPON(*)
        REAL*8          EPSM(6),  DEPS(6)
        REAL*8          SIGM(6),   SIGP(6)
        REAL*8          VIM(*),   VIP(*)
        REAL*8          DSIDEP(6,6)
C
        CHARACTER*16    COMPOR(*),     OPTION
        CHARACTER*8     TYPMOD(*)
        CHARACTER*(*)   FAMI
        LOGICAL         CP 
C
C       ----------------------------------------------------------------
C       VARIABLES LOCALES POUR LE REDECOUPAGE DU PAS DE TEMPS
C               TD      INSTANT T
C               TF      INSTANT T+DT
C               TEMD    TEMPERATURE A T
C               TEMF    TEMPERATURE A T+DT
C               DEPS    INCREMENT DE DEFORMATION TOTALE
C               VD      VARIABLES INTERNES A T    + INDICATEUR ETAT T
C               DSIDEPLO MATRICE DE COMPORTEMENT TANGENT A T+DT OU T
C               ICOMP           COMPTEUR POUR LE REDECOUPAGE DU PAS DE
C                                    TEMPS
C               RETURN1 EN CAS DE NON CONVERGENCE LOCALE
C       ----------------------------------------------------------------
C
        INTEGER         ICOMP
        INTEGER         NUMLC
        INTEGER         CODRET
C       ----------------------------------------------------------------
C       COMMONS POUR VARIABLES DE COMMANDE : CAII17 ET CARR01
        INTEGER NFPGMX
        PARAMETER (NFPGMX=10)
        INTEGER NFPG,JFPGL,DECALA(NFPGMX),KM,KP,KR,IREDEC
        COMMON /CAII17/NFPG,JFPGL,DECALA,KM,KP,KR,IREDEC
        REAL*8 INSTM1,INSTP1,TD1,TF1
        COMMON /CARR01/INSTM1,INSTP1,TD1,TF1
C       ----------------------------------------------------------------
C       ----------------------------------------------------------------
        COMMON /TDIM/   NDT  , NDI
C       ----------------------------------------------------------------
C       ----------------------------------------------------------------

      
C     NUMLC doit etre compris entre 1 et 100

      GO TO (101,102,103,104,105,106,107,108,109,110,111,112,113,114,
     &       115,116,117,118,119,120,121,122,123,124,125,126,127,128,
     &       129,130,131,132,133,134,135,136,137,138,139,140,141,142,
     &       143,144,145,146,147,148,149,150,151,152,153,154,155,156,
     &       157,158,159,160,161,162,163,164,165,166,167,168,169,170,
     &       171,172,173,174,175,176,177,178,179,180,181,182,183,184,
     &       185,186,187,188,189,190,191,192,193,194,195,196,197,198,
     &       199,200) NUMLC
      GOTO 9991
      
  101 CONTINUE
C     VMIS_ISOT_XXX, VISC_ISOT_XXX, ELAS
      CALL LC0001(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  102 CONTINUE
C     VMIS_CINE_LINE
      CALL LC0002(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  103 CONTINUE
C     VMIS_ECMI_XXXX
      CALL LC0003(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  104 CONTINUE
C     VMIS_CINX_CHAB/MEMO VISC_CINX_CHAB/MEMO, 
      CALL LC0004(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  105 CONTINUE
C     ENDO_FRAGILE
      CALL LC0005(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  106 CONTINUE
C     ENDO_ISOT_BETON
      CALL LC0006(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  107 CONTINUE
C     ENDO_ORTH_BETON
      CALL LC0007(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  108 CONTINUE
C     MAZARS
      CALL LC0008(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  109 CONTINUE
C     BETON_REGLE_PR
      CALL LC0009(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  110 CONTINUE
C     CZM_EXP_REG
      CALL LC0010(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  111 CONTINUE
C     CZM_LIN_REG
      CALL LC0011(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  112 CONTINUE
C     CZM_EXP
      CALL LC0012(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  113 CONTINUE
C     JOINT_BA
      CALL LC0013(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
  114 CONTINUE
C     ROUSSELIER
      CALL LC0014(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 115  CONTINUE
C     META_XXX
      CALL LC0015(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 116  CONTINUE
C     DRUCK_PRAGER
      CALL LC0016(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 117  CONTINUE
C     NORTON_HOFF
      CALL LC0017(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 118  CONTINUE
C     VISC_TAHERI
      CALL LC0018(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 119  CONTINUE
C     ELAS_HYPER
      CALL LC0019(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 120  CONTINUE
      CALL LC0020(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 121  CONTINUE
      CALL LC0021(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 122  CONTINUE
      CALL LC0022(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 123  CONTINUE
      CALL LC0023(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 124  CONTINUE
      CALL LC0024(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 125  CONTINUE
C     KIT_DDI : NE PAS UTILISER COMME EXEMPLE
      CALL LC0025(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,CP,EPSM,DEPS,SIGM,VIM,OPTION,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,NUMLC,DSIDEP,CODRET)
      GO TO 9992
 126  CONTINUE
      CALL LC0026(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 127  CONTINUE
      CALL LC0027(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 128  CONTINUE
      CALL LC0028(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 129  CONTINUE
      CALL LC0029(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 130  CONTINUE
C     TEMPD,TEMPF,TREF pour PLASTI, CAR APPEL POSSIBLE EN THM
C     NE PAS UTILISER COMME EXEMPLE
      CALL LC0030(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &        TEMPD,TEMPF,TREF,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 131  CONTINUE
      CALL LC0031(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 132  CONTINUE
C     TEMPD,TEMPF,TREF pour PLASTI, CAR APPEL POSSIBLE EN THM
C     NE PAS UTILISER COMME EXEMPLE
      CALL LC0032(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &        TEMPD,TEMPF,TREF,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 133  CONTINUE
C     TEMPD,TEMPF,TREF pour PLASTI, CAR APPEL POSSIBLE EN THM
C     NE PAS UTILISER COMME EXEMPLE
      CALL LC0033(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &        TEMPD,TEMPF,TREF,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 134  CONTINUE
      CALL LC0034(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 135  CONTINUE
      CALL LC0035(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 136  CONTINUE
      CALL LC0036(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 137  CONTINUE
      CALL LC0037(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 138  CONTINUE
      CALL LC0038(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 139  CONTINUE
      CALL LC0039(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 140  CONTINUE
      CALL LC0040(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 141  CONTINUE
      CALL LC0041(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 142  CONTINUE
      CALL LC0042(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 143  CONTINUE
      CALL LC0043(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 144  CONTINUE
      CALL LC0044(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 145  CONTINUE
      CALL LC0045(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 146  CONTINUE
      CALL LC0046(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 147  CONTINUE
      CALL LC0047(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 148  CONTINUE
      CALL LC0048(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 149  CONTINUE
      CALL LC0049(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 150  CONTINUE
      CALL LC0050(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 151  CONTINUE
      CALL LC0051(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 152  CONTINUE
      CALL LC0052(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 153  CONTINUE
      CALL LC0053(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 154  CONTINUE
      CALL LC0054(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 155  CONTINUE
      CALL LC0055(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 156  CONTINUE
      CALL LC0056(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 157  CONTINUE
      CALL LC0057(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 158  CONTINUE
      CALL LC0058(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 159  CONTINUE
      CALL LC0059(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 160  CONTINUE
      CALL LC0060(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 161  CONTINUE
      CALL LC0061(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 162  CONTINUE
      CALL LC0062(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 163  CONTINUE
      CALL LC0063(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 164  CONTINUE
      CALL LC0064(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 165  CONTINUE
      CALL LC0065(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 166  CONTINUE
      CALL LC0066(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 167  CONTINUE
      CALL LC0067(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 168  CONTINUE
      CALL LC0068(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 169  CONTINUE
      CALL LC0069(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 170  CONTINUE
      CALL LC0070(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 171  CONTINUE
      CALL LC0071(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 172  CONTINUE
      CALL LC0072(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 173  CONTINUE
      CALL LC0073(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 174  CONTINUE
      CALL LC0074(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 175  CONTINUE
      CALL LC0075(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 176  CONTINUE
      CALL LC0076(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 177  CONTINUE
      CALL LC0077(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 178  CONTINUE
      CALL LC0078(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 179  CONTINUE
      CALL LC0079(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 180  CONTINUE
      CALL LC0080(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 181  CONTINUE
      CALL LC0081(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 182  CONTINUE
      CALL LC0082(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 183  CONTINUE
      CALL LC0083(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 184  CONTINUE
      CALL LC0084(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 185  CONTINUE
      CALL LC0085(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 186  CONTINUE
      CALL LC0086(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 187  CONTINUE
      CALL LC0087(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 188  CONTINUE
      CALL LC0088(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 189  CONTINUE
      CALL LC0089(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992      
 190  CONTINUE
      CALL LC0090(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 191  CONTINUE
      CALL LC0091(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 192  CONTINUE
      CALL LC0092(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 193  CONTINUE
      CALL LC0093(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 194  CONTINUE
      CALL LC0094(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 195  CONTINUE
      CALL LC0095(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 196  CONTINUE
      CALL LC0096(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 197  CONTINUE
      CALL LC0097(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 198  CONTINUE
      CALL LC0098(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 199  CONTINUE
      CALL LC0099(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992
 200  CONTINUE
      CALL LC0100(FAMI,KPG,KSP,NDIM,IMATE,COMPOR,CRIT,INSTAM,
     &            INSTAP,EPSM,DEPS,SIGM,VIM,OPTION,ANGMAS,SIGP,VIP,
     &            TAMPON,TYPMOD,ICOMP,NVI,DSIDEP,CODRET)
      GO TO 9992      
 9991 CONTINUE
      CALL U2MESG('F','COMPOR1_43',0,' ',1,NUMLC,0,0.D0)
      
 9992 CONTINUE
      END
