      SUBROUTINE NTTYSE ( NBPASE, INPSCO, NOPASE, TYPESE, STYPSE )
C
C     THERMIQUE LINEAIRE - TYPE DE SENSIBILITE
C     *                    **      **
C     COMMANDE:  THER_LINEAIRE & THER_NON_LINE
C
C ----------------------------------------------------------------------
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ALGORITH  DATE 07/10/2008   AUTEUR PELLET J.PELLET 
C ======================================================================
C COPYRIGHT (C) 1991 - 2001  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
C (AT YOUR OPTION) ANY LATER VERSION.
C
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.
C
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
C    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
C ======================================================================
C RESPONSABLE GNICOLAS G.NICOLAS
C ----------------------------------------------------------------------
C IN  NBPASE  : NOMBRE DE PARAMETRES SENSIBLES
C IN  INPSCO  : STRUCTURE CONTENANT LA LISTE DES NOMS
C               VOIR LA DEFINITION DANS SEGICO
C IN  NOPASE  : NOM DU PARAMETRE SENSIBLE
C OUT TYPESE  : TYPE DE SENSIBILITE
C               -1 : DERIVATION EULERIENNE (VIA UN CHAMP THETA)
C                1 : CALCUL INSENSIBLE
C                2 : TEMPERATURE IMPOSEE
C                3 : PARAMETRE MATERIAU
C                4 : CARACTERISTIQUE ELEMENTAIRE (COQUES, ...)
C                5 : SOURCE
C                6 : FLUX LINEAIRE
C                7 : TEMPERATURE EXTERIEURE
C                8 : COEFFICIENT D'ECHANGE
C                9 : FLUX NON-LINEAIRE
C               10 : RAYONNEMENT
C OUT STYPSE  : SOUS-TYPE DE SENSIBILITE DANS LES CAS SUIVANTS :
C               . PARAMETRE MATERIAU
C               . RAYONNEMENT
C   -------------------------------------------------------------------
C     ASTER INFORMATIONS:
C       23/01/02 (OB): RAJOUT DE HECHP POUR TRAITER ECHANGE_PAROI.
C       12/03/02 (OB): RAJOUT DE FLUNL ET RAYO POUR TRAITER SENSIBILITE
C                      EN NON-LINEAIRE
C ----------------------------------------------------------------------
      IMPLICIT   NONE

C 0.1. ==> ARGUMENTS

      INTEGER NBPASE, TYPESE

      CHARACTER*(*) NOPASE, INPSCO
      CHARACTER*24 STYPSE

C 0.2. ==> COMMUNS
C 0.3. ==> VARIABLES LOCALES
C
      CHARACTER*6 NOMPRO
      PARAMETER ( NOMPRO = 'NTTYSE' )
C
      INTEGER NBMCRF, NBMCMX
      PARAMETER ( NBMCRF = 3, NBMCMX = 4 )
C
      INTEGER NBMOSI
      INTEGER NBMC(NBMCRF), NBMCSR
      CHARACTER*24 LIMOSI, LIVALE, LIMOFA
      CHARACTER*24 BLAN24
      CHARACTER*24 TYPEPS
      CHARACTER*24 COREFE(NBMCRF), MCSREF(NBMCMX)
C
C                    123456789012345678901234
      DATA BLAN24 / '                        ' /
C
C====
C 1. RECHERCHE GLOBALE DU TYPE
C               -1 : DERIVATION EULERIENNE (VIA UN CHAMP THETA)
C                1 : CALCUL INSENSIBLE
C                2 : CONDITION AUX LIMITES DE DIRICHLET
C                3 : PARAMETRE MATERIAU
C                4 : CARACTERISTIQUE ELEMENTAIRE (COQUES, ...)
C                0 : AUTRE TYPE, DEPENDANT DE LA COMMANDE
C====
C                12   345678   9012345678901234
      LIMOSI  = '&&'//NOMPRO//'_LIMOSI         '
      LIVALE  = '&&'//NOMPRO//'_LIVALE         '
      LIMOFA  = '&&'//NOMPRO//'_LIMOFA         '
C
      CALL PSTYSE ( NBPASE, INPSCO, NOPASE,
     &              TYPESE, TYPEPS, NBMOSI, LIMOSI, LIVALE, LIMOFA )
C
C====
C 2. TYPE DE SENSIBILITE
C====
C
      STYPSE = BLAN24
C
      IF ( TYPESE.EQ.3 ) THEN
C
        COREFE(1) = 'LAMBDA'
        NBMC(1) = 1
        MCSREF(1) = 'LAMBDA'
C
        COREFE(2) = 'RHO_CP'
        NBMC(2) = 1
        MCSREF(2) = 'RHO_CP'
C
        COREFE(3) = 'BETA'
        NBMC(3) = 1
        MCSREF(3) = 'BETA'
C
        NBMCSR = 3
C
        CALL PSTYSS ( NBMCSR, NBMC, COREFE, MCSREF,
     &                NBMOSI, LIVALE, NOPASE,
     &                STYPSE )
C
      ELSEIF ( TYPESE.EQ.0 ) THEN

        IF ( TYPEPS.EQ.'SOURCE' ) THEN
          TYPESE = 5
        ELSEIF ( TYPEPS.EQ.'FLUX' ) THEN
          TYPESE = 6
        ELSEIF ( TYPEPS.EQ.'T_EXT' ) THEN
          TYPESE = 7
        ELSEIF ((TYPEPS.EQ.'COEFH').OR.(TYPEPS.EQ.'HECHP')) THEN
          TYPESE = 8
        ELSEIF (TYPEPS.EQ.'FLUNL') THEN
          TYPESE = 9
        ELSEIF (TYPEPS.EQ.'RAYO') THEN
          TYPESE = 10
C
          COREFE(1) = 'SIGMA'
          NBMC(1) = 1
          MCSREF(1) = 'SIGMA'
C
          COREFE(2) = 'EPSILON'
          NBMC(2) = 1
          MCSREF(2) = 'EPSILON'
C
          COREFE(3) = 'TEMP_EXT'
          NBMC(3) = 1
          MCSREF(3) = 'TEMP_EXT'
C
          NBMCSR = 3
          CALL PSTYST ( NBMCSR, NBMC, COREFE, MCSREF,
     &                  NBMOSI, LIMOSI, LIVALE, LIMOFA,
     &                  STYPSE )
        ELSE

          CALL U2MESK('F', 'SENSIBILITE_9', 1, NOPASE)

        ENDIF

      ENDIF
C
C====
C 3. MENAGE
C====
C
      CALL JEDETR ( LIMOSI )
      CALL JEDETR ( LIVALE )
      CALL JEDETR ( LIMOFA )

      END
