      SUBROUTINE PRELOG(NDIM,NNO,AXI,GRAND,VFF,GEOMI,G,IW,IDFDE,
     &                  DEPLM,DEPLT,SIGM,LGPG,VIM,GN,FETA,XI,ME,
     &                  R,W,DFF,FM,FP,EPSML,DEPS,TN,RESI,PES)
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ALGORITH  DATE 25/01/2011   AUTEUR PROIX J-M.PROIX 
C ======================================================================
C COPYRIGHT (C) 1991 - 2011  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY  
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY  
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR     
C (AT YOUR OPTION) ANY LATER VERSION.                                   
C                                                                       
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT   
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF            
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              
C                                                                       
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,         
C   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.         
C ======================================================================
C TOLE CRP_21
C ----------------------------------------------------------------------
C  BUT:  CALCUL DES GRANDES DEFORMATIONS  LOG 2D (D_PLAN ET AXI) ET 3D
C     SUIVANT ARTICLE MIEHE APEL LAMBRECHT CMAME 2002
C ----------------------------------------------------------------------
C IN  NDIM    : DIMENSION DE L'ESPACE
C IN  NNO     : NOMBRE DE NOEUDS DE L'ELEMENT
C IN  AXI     : .TRUE. SI AXIS
C IN  GRAND   : .TRUE.
C IN  VFF     : VALEUR  DES FONCTIONS DE FORME AU POINT G
C IN  GEOMI   : COORDONNEES DES NOEUDS (CONFIGURATION INITIALE)
C IN  G       : NUMERO DU POINTS DE GAUSS
C IN  IW      : PTR. POIDS DES POINTS DE GAUSS
C IN  IDFDE   : PTR. DERIVEE DES FONCTIONS DE FORME ELEMENT DE REF.
C IN  DEPLM   : DEPLACEMENT EN T-
C IN  DEPLT   : DEPLACEMENT EN T+
C IN  SIGM    : CONTRAINTES DE CAUCHY EN T-
C IN  LGPG    : DIMENSION DU VECTEUR DES VAR. INTERNES POUR 1 PT GAUSS
C IN  VIM     : VARIABLES INTERNES EN T-
C OUT GN      : TERMES UTILES AU CALCUL DE TL DANS POSLOG
C OUT FETA    : TERMES UTILES AU CALCUL DE TL DANS POSLOG
C OUT XI      : TERMES UTILES AU CALCUL DE TL DANS POSLOG
C OUT ME      : TERMES UTILES AU CALCUL DE TL DANS POSLOG
C OUT R       : RAYON DU POINT DE GAUSS COURANT (EN AXI)
C OUT W       : POIDS DU POINT DE GAUSS COURANT (Y COMPRIS R EN AXI)
C OUT DFF     : DERIVEE DES FONCTIONS DE FORME (POINT DE GAUSS COURANT)
C OUT FM      : GRADIENT TRANSFORMATION EN T-
C OUT FP      : GRADIENT TRANSFORMATION EN T+
C OUT JM      : DET DU GRADIENT TRANSFORMATION EN T-
C OUT JP      : DET DU GRADIENT TRANSFORMATION EN T+
C OUT EPSML   : DEFORAMTIONS LOGARITHMIQUES EN T-
C OUT DEPS    : ACCROISSEEMENT DE DEFORMATIONS LOGARITHMIQUES
C OUT TN      : CONTRAINTES ASSOCIEES AUX DEF. LOGARITHMIQUES EN T-
C IN  RESI    : .TRUE. SI FULL_MECA/RAPH_MECA .FALSE. SI RIGI_MECA_TANG
C OUT PES     : OPERATEUR DE TRANSFORMATION TN (OU TP) EN PK2
C
      IMPLICIT NONE
      INTEGER I,NDIM,NNO,G,IW,IDFDE,LGPG,IVTN
      REAL*8 VFF(NNO),DEPLM(NDIM*NNO),DFF(NNO,*),SIGM(2*NDIM)
      REAL*8 GEOMI(*),W,PES(6,6),VIM(LGPG)
      REAL*8 FM(3,3),FP(3,3),DEPLT(NDIM*NNO)
      REAL*8 R,EPSML(6),EPSPL(6),PE(3,3,3,3),TBID(6)
      REAL*8 TN(6),DEPS(6),T(6),TL(3,3,3,3)
C      REAL*8 PES1(6,6),PK2(6),R8BID,PES0(6,6),RAC2
C      REAL*8 PEM(3,3,3,3),MEM(3,3,3,3),FETAM(4),XIM(3,3),GNM(3,3)
      REAL*8 GN(3,3),FETA(4),XI(3,3),ME(3,3,3,3)
      LOGICAL GRAND, AXI, RESI
C ---------------------------------------------------------------------

      IF (NDIM.EQ.3) AXI=.FALSE.
      CALL DFDMIP(NDIM,NNO,AXI,GEOMI,G,IW,VFF,IDFDE,R,W,DFF)

      CALL NMEPSI(NDIM,NNO,AXI,GRAND,VFF,R,DFF,DEPLM,FM,TBID)
      CALL NMEPSI(NDIM,NNO,AXI,GRAND,VFF,R,DFF,DEPLT,FP,TBID)
      

      CALL DEFLOG(NDIM,FM,EPSML,PE,GN,FETA,XI,ME, 0,T,TL)
      IF (RESI) THEN
         CALL DEFLOG(NDIM,FP,EPSPL,PE, GN, FETA, XI, ME , 0,T,TL)
         DO 35 I=1,6
            DEPS(I)=EPSPL(I)-EPSML(I)
 35      CONTINUE
      ELSE
         DO 34 I=1,6
            DEPS(I)=0.D0
 34      CONTINUE
      ENDIF
      
C     --------------------------------
C     CALCUL DES CONTRINTES TN INSTANT PRECEDENT
C     pour gagner tu temps : on stocke TN comme variable interne
C     --------------------------------
      IVTN=LGPG-6+1
      CALL DCOPY(2*NDIM,VIM(IVTN),1,TN,1)
      
C     --------------------------------
C     autre possibilite sans variable interne (couteuse en CPU)
C     --------------------------------
C      CALL R8INIR(6,0.D0,PK2,1)
C      CALL R8INIR(6,0.D0,TN,1)
C      CALL PK2SIG(NDIM,FM,JM,PK2,SIGM,-1)
C      CALL SYMT46(PEM,PES0)
C      CALL R8INIR(36,0.D0,PES1,1)
C      DO 36 I=1,6
C         PES1(I,I)=1.D0
C 36   CONTINUE
C      CALL MGAUSS('NFSP',PES0,PES1,6,6,6,R8BID,IRET)
C      RAC2=SQRT(2.D0)
C      DO 37 I=4,2*NDIM
C         PK2(I)=PK2(I)*RAC2
C 37   CONTINUE      
C      DO 38 I=1,6
C      DO 39 J=1,6
C      TN(I)=TN(I)+PK2(J)*PES1(J,I)
C 39   CONTINUE
C 38   CONTINUE
C     --------------------------------


      IF (RESI) THEN
         CALL SYMT46(PE,PES)
      ELSE
         CALL SYMT46(PE,PES)
      ENDIF
      
      END
