      SUBROUTINE DNDISS(IPARA,NMNBN,NMPLAS,NMDPLA,NMDDPL,NMPROX,DEPS,
     &           NEWNBN,NEWPLA,NEWDPL,NEWDDP,NEWZFG,DESPIT,
     &           DDISIT,DC1,DC2,DTG,NORMM,NORMN)

        IMPLICIT  NONE
C      -----------------------------------------------------------------
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF ELEMENTS  DATE 04/04/2011   AUTEUR COURTOIS M.COURTOIS 
C ======================================================================
C COPYRIGHT (C) 1991 - 2011  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
C (AT YOUR OPTION) ANY LATER VERSION.
C
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.
C
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
C    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
C ======================================================================
C
C     CALCUL DES EFFORTS
C     CALCUL DES COURBURES PLASTIQUES
C     CALCUL DE LA DISSIPATION PLASTIQUE
C
C IN  NMNBN : FORCE - BACKFORCE
C IN  NMPLAS : MOMENTS LIMITES DE PLASTICITE
C IN  NMDPLA : DERIVEES DES MOMENTS LIMITES DE PLASTICITE
C IN  NMDDPL : DERIVEES SECONDES DES MOMENTS LIMITES DE PLASTICITE
C IN  NMPROX : NMPROX > 0 : NBN DANS ZONE DE CRITIQUE
C IN  DC1 : MATRICE ELASTIQUE + CONSTANTES DE PRAGER
C IN  DC2 : MATRICE ELASTIQUE + CONSTANTES DE PRAGER
C IN  DTG : MATRICE TANGENTE
C IN  NORMM : NORME SUR LA FONCTION MP = F(N)
C IN  NORMN : NORME SUR LA FONCTION MP = F(N)
C
C IN/OUT IPARA : LISTES DE PARAMETRES DE TYPE ENTIER 
C                (ZIMAT NCRIT NEWIEF IER)
C
C OUT NEWNBN : FORCE - BACKFORCE A T+1
C OUT NEWPLA : MOMENTS LIMITES DE PLASTICITE A T+1
C OUT NEWDPL : DERIVEES DES MOMENTS LIMITES DE PLASTICITE A T+1
C OUT NEWDDP : DERIVEES SECONDES DES MOMENTS LIMITES DE PLASTICITE A T+1
C OUT NEWZFG : ZEROS ADIMENSIONNELS POUR LE CRITERE F ET G
C OUT NEWPRO : NEWPRO > 0 : NBN DANS ZONE DE CRITIQUE
C OUT DESPIT : INCREMENT DE DEFORMATION PLASTIQUE
C OUT DDISIT : INCREMENT DE DISSIPATION PLASTIQUE

      INTEGER NEWIEF, NMPROX(*)
      INTEGER NCRIT, ZIMAT, IER, CIEF
      INTEGER CIER, RESTZO, I, J, IPARA(4)

      REAL*8 NMNBN(*), NEWNBN(*), NMPLAS(2,*), NEWPLA(2,*)
      REAL*8 NMDPLA(2,*), NEWDPL(2,*), NMDDPL(2,*), NEWDDP(2,*)
      REAL*8 NEWZEF, NEWZEG, NEWZFG(2)
      REAL*8 DEPS(*),DC1(6,6),DC2(6,6),DTG(6,6)
      REAL*8 DESPIT(*),DDISIT, TDESPI(1,6),AUX(1)
      REAL*8 CNBN(6), CPLAS(2,3), CZEF, CZEG, CDEPS(6)
      REAL*8 CDTG(6,6), CDEPSP(6)
      REAL*8 NMP(6),NORMM,NORMN,RPARA(3)

      NEWZEF = NEWZFG(1)
      NEWZEG = NEWZFG(2)
      CZEF = 0.D0
      CZEG = 0.D0

      ZIMAT  = IPARA(1)
      NCRIT  = IPARA(2) 
      NEWIEF = IPARA(3)
      IER    = IPARA(4) 

      DO 20, J = 1,6
        CDEPS(J) = DEPS(J)
        DO 10, I = 1,6
          CDTG(I,J) = DTG(I,J)
 10     CONTINUE
 20   CONTINUE

      IF(NCRIT .EQ. -1) THEN
C     PREDICTEUR ELASTIQUE HORS CHAMP
        IER=3

      ELSEIF(NCRIT .EQ. 0) THEN
C     ELASTICITE = AUCUN CRITERE ACTIVE
        CALL R8INIR(6,0.0D0,DESPIT,1)
        CALL MATMUL(DTG,DEPS,6,6,1,NMP)

        DO 30, J = 1,6
          NEWNBN(J) = NMNBN(J) + NMP(J)
 30     CONTINUE

C     CALCUL DES MOMENTS LIMITES DE PLASTICITE
C     ET DES ZEROS DES CRITERES 
        CALL MPPFFN(ZIMAT,NEWNBN,NEWPLA,NEWZEF
     &                ,NEWZEG,NEWIEF,NORMM )

        IF(NEWIEF .GT. 0) THEN
          IER=2
        ELSE
          IER=0
        ENDIF
      ELSEIF(NCRIT .EQ. 1) THEN
C     CRITERE 1 ACTIVE

C     EST-ON PROCHE DU SOMMET DU CONE
        NMPROX(1) = RESTZO(ZIMAT,NMNBN,1,NORMM,NORMN)

        IF (NMPROX(1) .GT. 0) THEN
          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE
          CALL D1CRO2(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMDDPL,NMPROX,CNBN,
     &           CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,1)

          CZEF = RPARA(1)
          CZEG = RPARA(2)
          NORMM = RPARA(3)

          IF(CIER .EQ. 3) THEN
            RPARA(1) = CZEF
            RPARA(2) = CZEG
            RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE
            CALL D1CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &          CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,1)
            CZEF = RPARA(1)
            CZEG = RPARA(2)
            NORMM = RPARA(3)
          ENDIF
        ELSE
          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE     
          CALL D1CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &        CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,1)

          CZEF = RPARA(1)
          CZEG = RPARA(2)
          NORMM = RPARA(3)
        ENDIF

        DO 40, J = 1,6
          NEWNBN(J) = CNBN(J)
 40     CONTINUE

        DO 60, J = 1,3
          DO 50, I = 1,2
            NEWPLA(I,J) = CPLAS(I,J)
 50       CONTINUE
 60     CONTINUE

        NEWZEF = CZEF
        NEWZEG = CZEG
        NEWIEF = CIEF

        DO 100, J = 1,6
          DESPIT(J) = CDEPSP(J)
 100    CONTINUE

        IER = CIER

      ELSEIF(NCRIT .EQ. 2) THEN
C     CRITERE 2 ACTIVE     
        NMPROX(2) = RESTZO(ZIMAT,NMNBN,2,NORMM,NORMN)

        IF (NMPROX(2) .GT. 0) THEN
          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE     
          CALL D1CRO2(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMDDPL,NMPROX,CNBN,
     &           CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC2,2)
          CZEF = RPARA(1)
          CZEG = RPARA(2)
          NORMM = RPARA(3)

          IF(CIER .EQ. 3) THEN
            RPARA(1) = CZEF
            RPARA(2) = CZEG
            RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE       
            CALL D1CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &          CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC2,2)
            CZEF  = RPARA(1)
            CZEG  = RPARA(2)
            NORMM = RPARA(3)
          ENDIF
        ELSE

          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE     
          CALL D1CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &        CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC2,2)

          CZEF = RPARA(1)
          CZEG = RPARA(2)
          NORMM = RPARA(3)
        ENDIF

        DO 110, J = 1,6
          NEWNBN(J) = CNBN(J)
 110    CONTINUE

        DO 130, J = 1,3
          DO 120, I = 1,2
            NEWPLA(I,J) = CPLAS(I,J)
 120      CONTINUE
 130    CONTINUE

        NEWZEF = CZEF
        NEWZEG = CZEG
        NEWIEF = CIEF

        DO 180, J = 1,6
          DESPIT(J) = CDEPSP(J)
 180    CONTINUE

        IER = CIER

      ELSEIF(NCRIT .EQ. 12) THEN
C     CRITERES 1 ET 2 ACTIVES      
        NMPROX(1) = RESTZO(ZIMAT,NMNBN,1,NORMM,NORMN)
        NMPROX(2) = RESTZO(ZIMAT,NMNBN,2,NORMM,NORMN)

        IF ((NMPROX(1) .GT. 0) .AND. (NMPROX(2) .GT. 0)) THEN
          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DU MULTIPLICATEUR PLASTIQUE
C     ET DE L INCREMENT DE COURBURE PLASTIQUE     
          CALL D2CRO2(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMDDPL,NMPROX,CNBN,
     &             CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,DC2)

          CZEF = RPARA(1)
          CZEG = RPARA(2)
          NORMM = RPARA(3)

          IF(CIER .GT. 2) THEN
            RPARA(1) = CZEF
            RPARA(2) = CZEG
            RPARA(3) = NORMM

C     CALCUL DES MULTIPLICATEURS PLASTIQUES
C     ET DE L INCREMENT DE COURBURE PLASTIQUE       
            CALL D2CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &        CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,DC2)

            CZEF  = RPARA(1)
            CZEG  = RPARA(2)
            NORMM = RPARA(3)
          ENDIF
        ELSE

          RPARA(1) = CZEF
          RPARA(2) = CZEG
          RPARA(3) = NORMM

C     CALCUL DES MULTIPLICATEURS PLASTIQUES
C     ET DE L INCREMENT DE COURBURE PLASTIQUE     
          CALL D2CRIT(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMPROX,CNBN,
     &      CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,DC2)

          CZEF  = RPARA(1)
          CZEG  = RPARA(2)
          NORMM = RPARA(3)

          IF((CIER .EQ. 1).OR.(CIER .GT. 2)) THEN
            IF ((NMPROX(1) .GT. 0) .OR. (NMPROX(2) .GT. 0)) THEN

              RPARA(1) = CZEF
              RPARA(2) = CZEG
              RPARA(3) = NORMM

C     CALCUL DES MULTIPLICATEURS PLASTIQUES
C     ET DE L INCREMENT DE COURBURE PLASTIQUE         
              CALL D2CRO2(ZIMAT,NMNBN,NMPLAS,NMDPLA,NMDDPL,NMPROX,CNBN
     &               ,CPLAS,RPARA,CIEF,CDEPS,CDTG,CIER,CDEPSP,DC1,DC2)

              CZEF  = RPARA(1)
              CZEG  = RPARA(2)
              NORMM = RPARA(3)
            ENDIF
          ENDIF
        ENDIF

        DO 200, J = 1,6
          NEWNBN(J) = CNBN(J)
 200    CONTINUE

        DO 220, J = 1,3
          DO 210, I = 1,2
            NEWPLA(I,J) = CPLAS(I,J)
 210      CONTINUE
 220    CONTINUE

        NEWZEF = CZEF
        NEWZEG = CZEG
        NEWIEF = CIEF

        DO 260, J = 1,6
          DESPIT(J) = CDEPSP(J)
 260    CONTINUE

        IER = CIER

      ELSE
         CALL ASSERT(.FALSE.)

      ENDIF

      IF (IER .EQ. 0) THEN
C     CALCUL DES DERIVEES DES MOMENTS LIMITES DE PLASTICITE
C     CALCUL DES DERIVEES SECONDES DES MOMENTS LIMITES DE PLASTICITE    
        CALL D0MPFN(ZIMAT,NEWNBN,NEWDPL)
        CALL DDMPFN(ZIMAT,NEWNBN,NEWDDP)
      ENDIF

C     CALCUL DE LA DISSIPATION
      DO 265, J = 1,6
        TDESPI(1,J) = DESPIT(J)
 265  CONTINUE

      DO 270, J = 1,6
        NMP(J) = 0.5D0*(NMNBN(J) + NEWNBN(J))
 270  CONTINUE

      CALL MATMUL(TDESPI,NMP,1,6,1,AUX)

      DDISIT = AUX(1)

      NEWZFG(1) = NEWZEF
      NEWZFG(2) = NEWZEG

      IPARA(1) = ZIMAT
      IPARA(2) = NCRIT
      IPARA(3) = NEWIEF
      IPARA(4) = IER
      END
