      INTEGER FUNCTION INITS (OS, NOS, ETA)
C            CONFIGURATION MANAGEMENT OF EDF VERSION
C MODIF UTILIFOR  DATE 18/09/2007   AUTEUR DURAND C.DURAND 
C ======================================================================
C COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
C THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
C IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
C THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
C (AT YOUR OPTION) ANY LATER VERSION.
C
C THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
C WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
C MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
C GENERAL PUBLIC LICENSE FOR MORE DETAILS.
C
C YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
C ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
C   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
C ======================================================================
C
C UTILISEE SOUS NT POUR L'EVALUATION DE LA FONCTION D'ERREUR
C ERFC  (PROVIENT DE LA BIBLIOTHEQUE SLATEC)
C
C***BEGIN PROLOGUE  INITS
C***PURPOSE  DETERMINE THE NUMBER OF TERMS NEEDED IN AN ORTHOGONAL
C            POLYNOMIAL SERIES SO THAT IT MEETS A SPECIFIED ACCURACY.
C***LIBRARY   SLATEC (FNLIB)
C***CATEGORY  C3A2
C***TYPE      DOUBLE PRECISION (INITS-S, INITDS-D)
C***KEYWORDS  CHEBYSHEV, FNLIB, INITIALIZE, ORTHOGONAL POLYNOMIAL,
C             ORTHOGONAL SERIES, SPECIAL FUNCTIONS
C***AUTHOR  FULLERTON, W., (LANL)
C***DESCRIPTION
C
C  INITIALIZE THE ORTHOGONAL SERIES, REPRESENTED BY THE ARRAY OS, SO
C  THAT INITS IS THE NUMBER OF TERMS NEEDED TO INSURE THE ERROR IS NO
C  LARGER THAN ETA.  ORDINARILY, ETA WILL BE CHOSEN TO BE ONE-TENTH
C  MACHINE PRECISION.
C
C             INPUT ARGUMENTS --
C   OS     DOUBLE PRECISION ARRAY OF NOS COEFFICIENTS IN AN ORTHOGONAL
C          SERIES.
C   NOS    NUMBER OF COEFFICIENTS IN OS.
C   ETA    SINGLE PRECISION SCALAR CONTAINING REQUESTED ACCURACY OF
C          SERIES.
C
C***END PROLOGUE  INITDS
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 OS(*)
C***FIRST EXECUTABLE STATEMENT  INITDS
      CALL ASSERT(NOS .GE. 1)
C
      ERR = 0.D0
      DO 10 II = 1,NOS
        I = NOS + 1 - II
        ERR = ERR + ABS(OS(I))
        IF (ERR.GT.ETA) GO TO 20
   10 CONTINUE
C
   20 CONTINUE
C     ASSERT SI SERIE DE CHEBYSHEV TROP COURTE POUR LA PRECISION
      CALL ASSERT(I .NE. NOS)
      INITS = I
C
      END
